Package: bmstdr
Type: Package
Title: Bayesian Modeling of Spatio-Temporal Data with R
Version: 0.8.2
Authors@R: c(person(given="Sujit K.", family="Sahu", email="S.K.Sahu@soton.ac.uk", role = c("aut", "cre"),
    comment = c(ORCID = "0000-0003-2315-3598")), person(given="Duncan P.", family="Lee",
    email="Duncan.Lee@glasgow.ac.uk", role="aut"),  person(given="K. Shuvo", family="Bakar",
    email="shuvo.bakar@gmail.com", role="aut"))
URL: https://www.sujitsahu.com
Description: Fits, validates and compares a number of Bayesian models for
    spatial and space time point referenced and areal unit data. Model fitting
    is done using several packages: 'rstan', 'INLA', 'spBayes', 'spTimer',
    'spTDyn', 'CARBayes' and 'CARBayesST'. Model comparison is performed using
    the DIC and WAIC,  and K-fold cross-validation where the user is free
    to select their own subset of data rows for validation. Sahu (2022)
    <doi:10.1201/9780429318443> describes the methods in detail.
License: GPL-2
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.2
Imports: methods, spTimer, ggplot2, rstan, rstantools, spBayes,
        CARBayes, CARBayesST, spTDyn, MCMCpack, Rdpack, mnormt,
        inlabru, ggpubr
RdMacros: Rdpack
Depends: R (>= 3.4.0), Rcpp (>= 0.12.0)
Suggests: INLA (>= 22.5.7), coda, extraDistr, maps, xtable, MASS, loo,
        cowplot, ggrepel, sf, knitr, rmarkdown, testthat, ggspatial,
        interp, tidyr, RColorBrewer, magick, markdown
LinkingTo: BH (>= 1.66.0), Rcpp (>= 0.12.0), RcppEigen (>= 0.3.3.3.0),
        rstan (>= 2.18.1), StanHeaders (>= 2.18.0), RcppParallel
VignetteBuilder: knitr
SystemRequirements: GNU make
Additional_repositories: https://inla.r-inla-download.org/R/stable
BugReports: https://github.com/sujit-sahu/bmstdr/issues
NeedsCompilation: yes
Packaged: 2025-03-31 16:23:06 UTC; sks
Author: Sujit K. Sahu [aut, cre] (<https://orcid.org/0000-0003-2315-3598>),
  Duncan P. Lee [aut],
  K. Shuvo Bakar [aut]
Maintainer: Sujit K. Sahu <S.K.Sahu@soton.ac.uk>
Repository: CRAN
Date/Publication: 2025-03-31 17:30:06 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-01 03:13:13 UTC; unix
Archs: bmstdr.so.dSYM
