# Generated by roxygen2: do not edit by hand

S3method(coef,bmstdr)
S3method(fitted,bmstdr)
S3method(plot,bmstdr)
S3method(print,bmstdr)
S3method(residuals,bmstdr)
S3method(summary,bmstdr)
S3method(terms,bmstdr)
export(BCauchy)
export(Bcartime)
export(Bmchoice)
export(Bmoving_sptime)
export(Bnormal)
export(Bspatial)
export(Bsptime)
export(bmstdr_variogram)
export(calculate_validation_statistics)
export(fig11.13.plot)
export(fnc.delete.map.XYZ)
export(get_parameter_estimates)
export(get_validation_summaries)
export(getvalidrows)
export(hitandfalsealarm)
export(is.bmstdr)
export(materncov)
export(maternfun)
export(obs_v_pred_plot)
export(phichoice_sp)
export(phichoicep)
import(CARBayes)
import(CARBayesST)
import(MCMCpack)
import(Rcpp)
import(ggplot2)
import(graphics)
import(methods)
import(rstan)
import(spBayes)
import(spTimer)
import(stats)
importFrom(Rdpack,reprompt)
importFrom(ggpubr,ggarrange)
importFrom(inlabru,bru_safe_inla)
importFrom(mnormt,dmnorm)
importFrom(rstan,sampling)
importFrom(rstantools,rstan_config)
importFrom(utils,combn)
importFrom(utils,head)
useDynLib(bmstdr)
